local minute_abbreviation = "m";
local hour_abbreviation = "h";

function ArenaLive_Cooldown_Initialise(cooldown, parent)

	if ( not cooldown ) or ( not parent ) then
		ArenaLive_SystemMessage("CooldownFrame or Parent missing!", true);
		return;
	end
	
	parent.cooldown = cooldown;
	local prefix = cooldown:GetName();
	cooldown.text = _G[prefix.."Text"];
	cooldown.duration = 0;
	
	if ( cooldown.text ) then
	
		cooldown.noCooldownCount = 1;
		ArenaLive_Cooldown_UpdateTextSize(cooldown, parent);
		
	end
	
end


function ArenaLive_Cooldown_Set(cooldown, startTime, duration)

	if (not cooldown) then
		return;
	end

	local remainingDuration = duration - (GetTime() - startTime);
	
	ArenaLive_Cooldown_UpdateText(cooldown ,cooldown.text, remainingDuration)
	cooldown.duration = remainingDuration;
	cooldown:SetScript("OnUpdate", ArenaLive_Cooldown_OnUpdate);	
	cooldown:SetCooldown(startTime, duration);	
	
end


function ArenaLive_Cooldown_UpdateText(cooldown, cooldownText, duration)

	if (not cooldownText) then
		return;
	end

	if ( duration > 0 and ARENALIVE_DATABASE["options"]["showCooldownText"] ) then
		cooldown.noCooldownCount = 1;
		cooldownText:SetText(ArenaLive_Cooldown_FormatText(duration));
		cooldownText:Show();
	else
		cooldown.noCooldownCount = nil;
		cooldownText:Hide();
	end

end



function ArenaLive_Cooldown_UpdateTextSize(cooldown, parent)

	if ( (not cooldown) or (not parent) ) then
		return;
	end

	
	if ( cooldown.text ) then
	
		local _, sizeY = parent:GetSize();
		
		sizeY = math.floor(sizeY / 3 );
		

		local filename, fontHeight, flags = cooldown.text:GetFont()
		fontHeight = sizeY;
		cooldown.text:SetFont(filename, fontHeight, flags)
	
	end
end



function ArenaLive_Cooldown_OnUpdate(self, elapsed)

		self.duration = self.duration - elapsed;
		
		-- If duration is 0 the  cooldown has finished, if the frame is not visible it means that the cooldown isn't needed anymore (e.g. wenn CC is dispelled and PortraitOverlay therefore hidden). 
		if ( ( self.duration <= 0 ) ) then 
			self:SetScript("OnUpdate", nil)
			self.duration = 0;
			
			if (self.text) then
				ArenaLive_Cooldown_UpdateText(self, self.text, self.duration)
			end
			
		else
			
			if (self.text) then
				ArenaLive_Cooldown_UpdateText(self, self.text, self.duration)
			end	
		end
end



function ArenaLive_Cooldown_FormatText(cooldownTime)

	if ( not cooldownTime ) then
		return;
	end

	local cooldownTimeText;
	local timeType = "seconds";
	local decimal;
	
	-- Minutes
	if ( cooldownTime > 59 ) then
	
		cooldownTime = cooldownTime / 60;
		timeType = "minutes"
		
		-- Hours
		if ( cooldownTime > 60 ) then
			cooldownTime = cooldownTime / 60;
			timeType = "hours"
		end
		
		-- We need to round up or down correctly on this one.
		decimal = math.floor(cooldownTime * 10);
		decimal =  tonumber(string.sub(decimal, -1));
		
		if ( decimal < 5 ) then
			cooldownTime = math.floor(cooldownTime);
		else
			cooldownTime = math.ceil(cooldownTime);
		end

	else
		if (cooldownTime < 10 and math.floor(cooldownTime) > 0 ) then
			decimal = (math.floor(cooldownTime*10));
			cooldownTime = string.sub(decimal, 1, -2);
			cooldownTime = cooldownTime..DECIMAL_SEPERATOR;
			cooldownTime = cooldownTime..string.sub(decimal, -1);
			return cooldownTime;
		end
			
		if (math.floor(cooldownTime) == 0 ) then
			cooldownTime = string.sub(cooldownTime, 1, 3);
		else
			cooldownTime = math.floor(cooldownTime)
		end
	end
	
	if ( timeType == "hours" ) then
		cooldownTimeText = cooldownTime..hour_abbreviation;
	elseif ( timeType == "minutes" ) then 
		cooldownTimeText = cooldownTime..minute_abbreviation;
	else
		if ( tonumber(cooldownTime) <= 0 ) then
			cooldownTimeText = "";
		else
			cooldownTimeText = cooldownTime;
		end
	end
	
	
	return cooldownTimeText;
end