-- ICONS THAT SHOW TRINKET, RACE, ETC.
function ArenaLive_Icon_Initialise (self, addonName, frameType, iconID, texture, cooldown, parent)
	
	-- Create a shortcut for the parent frame.
	if (not parent ) then
		parent = self:GetParent();
	end
	parent["icon"..iconID] = self;
	
	self.addonName = addonName;
	self.frameType = frameType;
	self.texture = texture;
	ArenaLive_Cooldown_Initialise(cooldown, self);
	
	
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("UNIT_FACTION");
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
	self:SetScript("OnEvent", ArenaLive_Icon_OnEvent);
end


function ArenaLive_Icon_UpdateUnit (self, unit)

	self.unit = unit;
	ArenaLive_Icon_Update(self)
end



function ArenaLive_Icon_Update (self)
	
	if (not self.unit) then
		return;
	end
	
	-- First we reset Cooldowns and variables in case the icon type has been changed.
	ArenaLive_Cooldown_Set(self.cooldown, GetTime(), 0)
	self.sharedSpellID = nil;
	self.sharedCD = nil;
	self.specID = nil;
	self.race = nil;
	self.gender = nil;
	self.class = nil;
	self.isDispel = nil;
	self.specIcon = nil;
	
	local id = self:GetID();
	local iconType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["icon"..id.."Type"];

	
	if ( iconType == "race" ) then
		ArenaLive_Icon_UpdateRace(self);
	elseif (iconType == "class" ) then
		ArenaLive_Icon_UpdateClass(self);
	elseif (iconType == "trinket" ) then
		ArenaLive_Icon_UpdateTrinket(self);
	elseif (iconType == "racial" ) then
		ArenaLive_Icon_UpdateRacial(self);
	elseif (iconType == "specialisation" ) then
		ArenaLive_Icon_UpdateSpec(self);
	elseif (iconType == "interruptOrDispel" ) then
		ArenaLive_Icon_UpdateInterruptOrDispel(self);
	elseif (iconType == "reaction" ) then
		ArenaLive_Icon_UpdateReaction(self);
	else
	
	end
end



function ArenaLive_Icon_OnEvent (self, event, ...)

	if (not self.unit) then
		return;
	end
	
	local id = self:GetID();
	local iconType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["icon"..id.."Type"];

	if ( event == "PLAYER_ENTERING_WORLD" ) and ( self.unit == "player" ) and ( iconType == "specialisation" ) then
	
		-- We only need this once at startup, because "PLAYER_SPECIALIZATION_CHANGED" fires too early to get SpecData when logging in.
		self:UnregisterEvent("PLAYER_ENTERING_WORLD");
		ArenaLive_Icon_UpdateSpec(self);
		
	elseif ( ( event == "PLAYER_SPECIALIZATION_CHANGED" ) and ( self.unit == "player" ) and ( iconType == "specialisation" ) ) then
		
		ArenaLive_Icon_UpdateSpec(self)
		
	elseif ( ( event == "UNIT_SPELLCAST_SUCCEEDED" ) and ( iconType == "trinket" or iconType == "racial" ) ) then
	
		local unit = select(1, ...);
		local spellID = select(5, ...);
		
		if (unit == self.unit) then
				
			if ( spellID == self.spellID ) then	
				if ( UnitIsEnemy("player", unit) and iconType == "trinket" ) then
					local name = GetUnitName(unit, true)
					SendChatMessage("TRINKET USED: "..tostring(name) ,"PARTY");
				end
					
				ArenaLive_Cooldown_Set(self.cooldown, GetTime(), self.spellCD);
					
			elseif ( spellID == self.sharedSpellID ) then
				ArenaLive_Cooldown_Set(self.cooldown, GetTime(), self.sharedCD)
			end

			
		end
	elseif ( ( event == "UNIT_SPELLCAST_SUCCEEDED" ) and ( iconType == "interruptOrDispel" and not self.isDispel) ) then
	
		local unit = select(1, ...);
		local spellID = select(5, ...);
		local unitType = string.match(self.unit, "^([a-z]+)[0-9]+$") or self.unit;
		local unitNumber = tonumber(string.match(self.unit, "^[a-z]+([0-9]+)$"));
		local unitPet;
		
		if ( unitNumber ) then
			unitPet = unitType.."pet"..unitNumber;
		else
			unitPet = unitType.."pet";
		end
		
		if ( unit == self.unit or unit == unitPet ) then
			-- First we need to check if the spellID storage is a number or a table, because spell lock from warlocks has lots of spellIDs.
			local varType = type(self.spellID);
			
			if ( varType == "table" ) then
				local match;
				
				for key, value in ipairs(self.spellID) do
					if ( value == spellID ) then
						match = spellID;
					end
				end
				
				if ( match ) then
					ArenaLive_Cooldown_Set(self.cooldown, GetTime(), self.spellCD[spellID])
				end
			else
				if ( spellID == self.spellID ) then	
						
					ArenaLive_Cooldown_Set(self.cooldown, GetTime(), self.spellCD);
				end				
			end
		end
		
	elseif ( event == "UNIT_FACTION" and select(1, ...) == self.unit and iconType == "reaction" ) then
		ArenaLive_Icon_Update(self);
	elseif ( event == "COMBAT_LOG_EVENT_UNFILTERED" and iconType == "interruptOrDispel" and self.isDispel ) then
		
		local event2 = select(2, ...);
		local sourceName = select(5, ...);
		local spellID = select(15, ...);
		local name, realm = UnitName(self.unit);
		if ( realm and realm ~= "" ) then
			name = name.."-"..realm;
		end

		if ( ( event2 == "SPELL_DISPEL" ) and (sourceName == name) and (spellID == self.spellID) ) then
			ArenaLive_Cooldown_Set(self.cooldown, GetTime(), self.spellCD);
		end
	end

end

function ArenaLive_Icon_UpdateRace (self)

	if ( not self.unit ) then
		return;
	end

	local _, race = UnitRace(self.unit);
	local gender = UnitSex(self.unit);
	local isPlayer = UnitIsPlayer(self.unit);
	if ( ( ( race ~= self.race ) or ( gender ~= self.gender ) ) and ( isPlayer ) ) then
		self.race = race;
		self.gender = gender;
		
		if (gender > 1 ) then
			self.texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CHARACTERCREATE-RACES");
		else
			self.texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Races2");
		end
		
		self.texture:SetTexCoord(unpack(RACE_GENDER_ICON_TCOORDS[race][gender]));
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		self.texture:SetTexCoord(0, 1, 0, 1);
	end

end



function ArenaLive_Icon_UpdateClass (self)

	if ( not self.unit ) then
			return;
	end
	
	local _, class = UnitClass(self.unit);
	
	if  ( class ~= self.class ) and ( class ) then
		self.class = class;
		self.texture:SetTexture("Interface\\Glues\\CharacterCreate\\UI-CharacterCreate-Classes");
		self.texture:SetTexCoord(unpack(CLASS_ICON_TCOORDS[class]));
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		self.texture:SetTexCoord(0, 1, 0, 1);
	end	

end



function ArenaLive_Icon_UpdateTrinket (self)

	if ( not self.unit ) then
			return;
	end
	
	local isPlayer = UnitIsPlayer(self.unit);
	
	if ( isPlayer ) then
		local _, faction = UnitFactionGroup(self.unit);
		local _, race = UnitRace(self.unit);
		local _, class = UnitClass(self.unit);
			
		if ( faction == "Alliance" ) then
			self.texture:SetTexture("Interface\\ICONS\\INV_Jewelry_TrinketPVP_01");
		else
			self.texture:SetTexture("Interface\\ICONS\\INV_Jewelry_TrinketPVP_02");
		end
			
		self.spellID =  ARENALIVE_SPELLS.trinket
		self.spellCD = 120;
			
		-- Some racials have a sharedCD with the PvP-Trinket, so we check if this is the case.
		if (race == "Draenei") or (race == "Orc") or (race =="BloodElf") then
			if ( ARENALIVE_SPELLS.racials[race][class][3] > 0 ) then
				self.sharedSpellID = ARENALIVE_SPELLS.racials[race][class][1];
				self.sharedCD = ARENALIVE_SPELLS.racials[race][class][3];
			end
		else
			if ( ARENALIVE_SPELLS.racials[race][3] > 0 ) then
				self.sharedSpellID = ARENALIVE_SPELLS.racials[race][1];
				self.sharedCD = ARENALIVE_SPELLS.racials[race][3];		
			end
		end
		
		local currentCD = ArenaLive_IconCooldownTracking_GetSpellCooldown(self);
		
		if ( currentCD ) then
			local startTime = GetTime() - ( self.spellCD - currentCD );
			ArenaLive_Cooldown_Set(self.cooldown, startTime, self.spellCD);
		end
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
	end
	
	self.texture:SetTexCoord(0, 1, 0, 1);
end



function ArenaLive_Icon_UpdateRacial (self)

	if ( not self.unit ) then
			return;
	end
	local isPlayer = UnitIsPlayer(self.unit);
	
	if ( isPlayer ) then
		local _, race = UnitRace(self.unit);
		local _, class = UnitClass(self.unit);
		local sharedCD;
			
		if (race == "Draenei") or (race == "Orc") or (race == "BloodElf") then
				self.spellID = ARENALIVE_SPELLS.racials[race][class][1];
				self.spellCD = ARENALIVE_SPELLS.racials[race][class][2];
				sharedCD = ARENALIVE_SPELLS.racials[race][class][3];
		else
				self.spellID = ARENALIVE_SPELLS.racials[race][1];
				self.spellCD = ARENALIVE_SPELLS.racials[race][2];
				sharedCD = ARENALIVE_SPELLS.racials[race][3];
		end
			
			
		local _, _, icon = GetSpellInfo(self.spellID);
		self.texture:SetTexture(icon);
			
		-- Again Some racials have a sharedCD...
		if (sharedCD > 0 ) then
			self.sharedSpellID = ARENALIVE_SPELLS.trinket;
			self.sharedCD = sharedCD;
		end
		
		local currentCD = ArenaLive_IconCooldownTracking_GetSpellCooldown(self);
		if ( currentCD ) then
			local startTime = GetTime() - ( self.spellCD - currentCD );
			ArenaLive_Cooldown_Set(self.cooldown, startTime, self.spellCD);
		end
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
	end
	
	self.texture:SetTexCoord(0, 1, 0, 1);
end



function ArenaLive_Icon_UpdateSpec (self)

	if ( not self.unit ) then
			return;
	end	
	local specID;
	
	local unitType = string.match(self.unit, "^([a-z]+)[0-9]+$") or self.unit;
	local unitNumber = tonumber(string.match(self.unit, "^[a-z]+([0-9]+)$"));
	
	if (unitType == "player" ) then	
			
		specID = GetSpecialization();
			
		if ( specID ) then
				
			local _, _, _, icon = GetSpecializationInfo(specID);
			
			if ( icon ) then
				self.texture:SetTexture(icon);
				self.specIcon = icon;
			else
				self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			end
				
		else
			self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
		end
	elseif (unitType == "arena" ) then
		local numOpps = GetNumArenaOpponentSpecs();
		
		if ( numOpps and unitNumber and unitNumber <= numOpps ) then
		
			local specID = GetArenaOpponentSpec(unitNumber);
			
			if (specID > 0) then
			
				local _, _, _, icon = GetSpecializationInfoByID(specID);
				
				if ( icon ) then
					self.texture:SetTexture(icon);
					self.specIcon = icon;
				else
					self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
				end
				
			else
				self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
			end
		
		else
			self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark");
		end
	end
		
	self.texture:SetTexCoord(0, 1, 0, 1);

end


function ArenaLive_Icon_UpdateInterruptOrDispel (self)

	if ( not self.unit ) then
			return;
	end	

	local _, class = UnitClass(self.unit);
	local isPlayer = UnitIsPlayer(self.unit);
	if ( class and isPlayer ) then
		local varType = type(ARENALIVE_SPELLS["dispelsOrInterrupts"][class][1]);
		if ( varType == "table" ) then
			-- Warlocks have more then 1 spellID for their interrupt, so we need to store this information in a table.
			self.spellID = {};
			self.spellCD = {};
			self.isDispel = ARENALIVE_SPELLS["dispelsOrInterrupts"][class][1][3];
			
			for key, value in ipairs(ARENALIVE_SPELLS["dispelsOrInterrupts"][class]) do
				local spellID = ARENALIVE_SPELLS["dispelsOrInterrupts"][class][key][1];
				local spellCD = ARENALIVE_SPELLS["dispelsOrInterrupts"][class][key][2];
				local _, _, icon = GetSpellInfo(spellID);
				
				self.spellID[key] = spellID;
				self.spellCD[spellID] = spellCD;
				
				self.texture:SetTexture(icon);
			end

			local currentCD = ArenaLive_IconCooldownTracking_GetSpellCooldown(self);
			if ( currentCD ) then
				local startTime = GetTime() - ( ARENALIVE_SPELLS["dispelsOrInterrupts"][class][1][2] - currentCD );
				ArenaLive_Cooldown_Set(self.cooldown, startTime, ARENALIVE_SPELLS["dispelsOrInterrupts"][class][1][2]);
			end
			
		else
			local spellID, cooldown, isDispel = unpack(ARENALIVE_SPELLS["dispelsOrInterrupts"][class]);
			self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
			self.texture:SetTexCoord(0, 1, 0, 1);	
			self.spellID =  spellID
			self.spellCD = cooldown;
			self.isDispel = isDispel;

			local _, _, icon = GetSpellInfo(self.spellID);
			self.texture:SetTexture(icon);
			
			local currentCD = ArenaLive_IconCooldownTracking_GetSpellCooldown(self);
			if ( currentCD ) then
				local startTime = GetTime() - ( self.spellCD - currentCD );
				ArenaLive_Cooldown_Set(self.cooldown, startTime, self.spellCD);
			end
		end
	else
		self.texture:SetTexture("Interface\\Icons\\INV_Misc_QuestionMark")
		self.texture:SetTexCoord(0, 1, 0, 1);
		self.spellID =  nil;
		self.spellCD = nil;
	end
end



function ArenaLive_Icon_UpdateReaction (self)

	if ( not self.unit ) then
			return;
	end	

	local red, green, blue = UnitSelectionColor(self.unit);

	if ( not UnitPlayerControlled(self.unit) and UnitIsTapped(self.unit) and not UnitIsTappedByPlayer(self.unit) and not UnitIsTappedByAllThreatList(self.unit) ) then
		self.texture:SetTexture(0.5, 0.5, 0.5, 1);
	else
		self.texture:SetTexture(red, green, blue, 1);
	end
	
	

end

-- CooldownTracking starts here
local trinketCooldownTable = {};
local racialCooldownTable = {};
local interruptOrDispelCooldownTable = {};
function ArenaLive_IconCooldownTracking_OnEvent(self, event, ...)

	-- NPCs with abilities would cause errors, because the Icons don't get set up for them.
	if ( event == "UNIT_SPELLCAST_SUCCEEDED" ) then

		local unit = select(1, ...);
		local spellID = select(5, ...);
		local cooldown;
		local sharedCooldown;
		local _, race = UnitRace(unit);
		local _, class = UnitClass(unit);
		
		if ( not UnitIsPlayer(unit) ) then
			return;
		end
		ArenaLive_SystemMessage(unit..":"..spellID, true);
		if ( spellID == ARENALIVE_SPELLS["trinket"] ) then
			
			cooldown = 120;
			
			ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, cooldown, trinketCooldownTable, nil);
			
			-- SharedCD with PvP-Trinket
			if (race == "Draenei") or (race == "Orc") or (race =="BloodElf") then
				sharedCooldown = ARENALIVE_SPELLS["racials"][race][class][3]
			else
				sharedCooldown = ARENALIVE_SPELLS["racials"][race][3]
			end
			
			if ( sharedCooldown > 0 ) then
				ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, sharedCooldown, racialCooldownTable, nil);
			end
			
			return;
		end
		
		if (race == "Draenei") or (race == "Orc") or (race =="BloodElf") then
			if (spellID == ARENALIVE_SPELLS["racials"][race][class][1] ) then
				cooldown = ARENALIVE_SPELLS["racials"][race][class][2];
				
				ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, cooldown, racialCooldownTable, nil);
				
				sharedCooldown = ARENALIVE_SPELLS["racials"][race][class][3];
				if ( sharedCooldown > 0 ) then
					ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, sharedCooldown, trinketCooldownTable, nil);
				end
				
				return;
			end
			
		else
			if (spellID == ARENALIVE_SPELLS["racials"][race][1] ) then
				
				cooldown = ARENALIVE_SPELLS["racials"][race][2];
				
				ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, cooldown, racialCooldownTable, nil);
				
				sharedCooldown = ARENALIVE_SPELLS["racials"][race][3];
				if ( sharedCooldown > 0 ) then
					ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, sharedCooldown, trinketCooldownTable, nil);
				end
				
				return;
			end
		end
		
		local varType = type(ARENALIVE_SPELLS["dispelsOrInterrupts"][class][1]);
		local match;
		if ( varType == "table" ) then
			
			for key, value in ipairs(ARENALIVE_SPELLS["dispelsOrInterrupts"][class]) do	
				if ( ARENALIVE_SPELLS["dispelsOrInterrupts"][class][key][1] == spellID and not ARENALIVE_SPELLS["dispelsOrInterrupts"][class][key][3] ) then
					ArenaLive_SystemMessage("UnitID: "..unit, true);
					match = key;
					break;
				end					
			end
			
			if ( match ) then
				cooldown = ARENALIVE_SPELLS["dispelsOrInterrupts"][class][match][2];
				ArenaLive_SystemMessage("Cooldown: "..cooldown, true);
				ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, cooldown, interruptOrDispelCooldownTable, nil);
			end
			
			return;
		else
		
			if ( spellID == ARENALIVE_SPELLS["dispelsOrInterrupts"][class][1] and not ARENALIVE_SPELLS["dispelsOrInterrupts"][class][3] ) then
				
				cooldown = ARENALIVE_SPELLS["dispelsOrInterrupts"][class][2];
				ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, cooldown, interruptOrDispelCooldownTable, nil);
				return;
			end
		
		end
		

	elseif ( event == "COMBAT_LOG_EVENT_UNFILTERED" ) then

		local event2 = select(2, ...);
		local sourceName = select(5, ...);
		local spellID = select(15, ...);
		local match;
		
		if ( event2 == "SPELL_DISPEL" ) then
			for key, value in pairs(ARENALIVE_SPELLS["dispelsOrInterrupts"]) do
				if ( spellID == ARENALIVE_SPELLS["dispelsOrInterrupts"][key][1] ) then
					match = ARENALIVE_SPELLS["dispelsOrInterrupts"][key][2]
				end
			
			end		
			if ( match ) then
				ArenaLive_IconCooldownTracking_AddCooldownToTable(nil, match, interruptOrDispelCooldownTable, sourceName);
			end
		end
	end
end

function ArenaLive_IconCooldownTracking_UpdateSpellCooldowns (self, elapsed)

	for key, value in pairs(trinketCooldownTable) do		
		local newValue = value - elapsed
		if ( newValue > 0 ) then
			trinketCooldownTable[key] = newValue;
		else
			trinketCooldownTable[key] = nil;
		end
	end
	
	for key, value in pairs(racialCooldownTable) do		
		local newValue = value - elapsed
		if ( newValue > 0 ) then
			racialCooldownTable[key] = newValue;
		else
			racialCooldownTable[key] = nil;
		end
	end

	for key, value in pairs(interruptOrDispelCooldownTable) do		
		local newValue = value - elapsed
		if ( newValue > 0 ) then
			interruptOrDispelCooldownTable[key] = newValue;
		else
			interruptOrDispelCooldownTable[key] = nil;
		end
	end
end

function ArenaLive_IconCooldownTracking_AddCooldownToTable(unit, cooldown, cooldownTable, name)
	
	local realm;
	
	if ( (not unit and not name) or (not cooldown) or (not cooldownTable) ) then
		return;
	end
	if ( not name ) then
		name, realm = UnitName(unit);
	end
	
	if ( realm and realm ~= "" ) then
		name = name.."-"..realm;
	end
	
	cooldownTable[name] = cooldown;
	ArenaLive_SystemMessage("Name: "..name, true);
end

function ArenaLive_IconCooldownTracking_GetSpellCooldown(self)

	local id = self:GetID();
	local name, realm = UnitName(self.unit);
	local iconType = ARENALIVE_DATABASE[self.addonName]["frameTypes"][self.frameType]["icon"..id.."Type"];
	
	if ( realm and realm ~= "" ) then
		name = name.."-"..realm;
	end
	
	local cooldownTable;
	
	if ( iconType == "trinket" ) then
		cooldownTable = trinketCooldownTable;
	elseif ( iconType == "racial" ) then
		cooldownTable = racialCooldownTable;
	elseif ( iconType == "interruptOrDispel" ) then
		cooldownTable = interruptOrDispelCooldownTable;
	end
	
	if (cooldownTable and cooldownTable[name] ) then
		return cooldownTable[name];
	else
		return 0;
	end
	
end


function ArenaLive_IconCooldownTracking_GetGarbageInfo()
	local msg;
	
	for key, value in pairs(trinketCooldownTable) do
		msg = key.." = ".. value;
		ArenaLive_SystemMessage(msg, true);
	end
	
	for key, value in pairs(racialCooldownTable) do
		msg = key.." = ".. value;
		ArenaLive_SystemMessage(msg, true);
	end

	for key, value in pairs(interruptOrDispelCooldownTable) do
		msg = key.." = ".. value;
		ArenaLive_SystemMessage(msg, true);
	end

end